@@ -42,10 +42,7 @@
 
 #include "e-book-backend-mapi.h"
 
-static void e_book_backend_mapi_authenticator_init (ESourceAuthenticatorInterface *iface);
-
-G_DEFINE_TYPE_WITH_CODE (EBookBackendMAPI, e_book_backend_mapi, E_TYPE_BOOK_BACKEND,
-	G_IMPLEMENT_INTERFACE (E_TYPE_SOURCE_AUTHENTICATOR, e_book_backend_mapi_authenticator_init))
+G_DEFINE_TYPE (EBookBackendMAPI, e_book_backend_mapi, E_TYPE_BOOK_BACKEND)
 
 struct _EBookBackendMAPIPrivate
 {
@@ -348,16 +345,19 @@ ebbm_maybe_invoke_cache_update (EBookBackendMAPI *ebma)
 
 static ESourceAuthenticationResult
 ebbm_connect_user (EBookBackendMAPI *ebma,
+		   const ENamedParameters *credentials,
+		   gboolean update_connection_status,
 		   GCancellable *cancellable,
-		   const GString *password,
 		   GError **error)
 {
 	EBookBackendMAPIPrivate *priv = ebma->priv;
 	EMapiConnection *old_conn;
 	CamelMapiSettings *settings;
+	ESource *source;
 	GError *mapi_error = NULL;
 
 	settings = ebbm_get_collection_settings (ebma);
+	source = e_backend_get_source (E_BACKEND (ebma));
 
 	if (!e_backend_get_online (E_BACKEND (ebma))) {
 		ebbm_notify_connection_status (ebma, FALSE);
@@ -369,8 +369,12 @@ ebbm_connect_user (EBookBackendMAPI *ebma,
 		}
 
 		e_book_backend_mapi_lock_connection (ebma);
+		if (update_connection_status)
+			e_source_set_connection_status (source, E_SOURCE_CONNECTION_STATUS_CONNECTING);
 
 		if (g_cancellable_set_error_if_cancelled (cancellable, error)) {
+			if (update_connection_status)
+				e_source_set_connection_status (source, E_SOURCE_CONNECTION_STATUS_DISCONNECTED);
 			e_book_backend_mapi_unlock_connection (ebma);
 			return E_SOURCE_AUTHENTICATION_ERROR;
 		}
@@ -381,11 +385,11 @@ ebbm_connect_user (EBookBackendMAPI *ebma,
 		priv->conn = e_mapi_connection_new (
 			e_book_backend_get_registry (E_BOOK_BACKEND (ebma)),
 			camel_mapi_settings_get_profile (settings),
-			password, cancellable, &mapi_error);
+			credentials, cancellable, &mapi_error);
 		if (!priv->conn) {
 			priv->conn = e_mapi_connection_find (camel_mapi_settings_get_profile (settings));
 			if (priv->conn && !e_mapi_connection_connected (priv->conn))
-				e_mapi_connection_reconnect (priv->conn, password, cancellable, &mapi_error);
+				e_mapi_connection_reconnect (priv->conn, credentials, cancellable, &mapi_error);
 		}
 
 		if (old_conn)
@@ -401,6 +405,8 @@ ebbm_connect_user (EBookBackendMAPI *ebma,
 
 			if (is_network_error)
 				mapi_error_to_edb_error (error, mapi_error, E_DATA_BOOK_STATUS_OTHER_ERROR, NULL);
+			if (update_connection_status)
+				e_source_set_connection_status (source, E_SOURCE_CONNECTION_STATUS_DISCONNECTED);
 			e_book_backend_mapi_unlock_connection (ebma);
 
 			if (mapi_error)
@@ -411,6 +417,8 @@ ebbm_connect_user (EBookBackendMAPI *ebma,
 			return is_network_error ? E_SOURCE_AUTHENTICATION_ERROR : E_SOURCE_AUTHENTICATION_REJECTED;
 		}
 
+		if (update_connection_status)
+			e_source_set_connection_status (source, E_SOURCE_CONNECTION_STATUS_CONNECTED);
 		e_book_backend_mapi_unlock_connection (ebma);
 
 		ebbm_notify_connection_status (ebma, TRUE);
@@ -440,10 +448,9 @@ e_book_backend_mapi_ensure_connected (EBookBackendMAPI *ebma,
 	settings = ebbm_get_collection_settings (ebma);
 
 	if (!camel_mapi_settings_get_kerberos (settings) ||
-	    ebbm_connect_user (ebma, cancellable, NULL, &local_error) != E_SOURCE_AUTHENTICATION_ACCEPTED) {
-		e_backend_authenticate_sync (
-			E_BACKEND (ebma),
-			E_SOURCE_AUTHENTICATOR (ebma),
+	    ebbm_connect_user (ebma, NULL, TRUE, cancellable, &local_error) != E_SOURCE_AUTHENTICATION_ACCEPTED) {
+		e_backend_credentials_required_sync (E_BACKEND (ebma),
+			E_SOURCE_CREDENTIALS_REASON_REQUIRED, NULL, 0, NULL,
 			cancellable, &local_error);
 	}
 
@@ -466,10 +473,10 @@ e_book_backend_mapi_maybe_disconnect (EBookBackendMAPI *ebma,
 	if (!mapi_error || !ebma->priv->conn)
 		return;
 
-	if (g_error_matches (mapi_error, E_MAPI_ERROR, MAPI_E_NETWORK_ERROR) ||
+	if (g_error_matches (mapi_error, E_MAPI_ERROR, ecRpcFailed) ||
 	    g_error_matches (mapi_error, E_MAPI_ERROR, MAPI_E_CALL_FAILED)) {
 		e_mapi_connection_disconnect (ebma->priv->conn,
-			!g_error_matches (mapi_error, E_MAPI_ERROR, MAPI_E_NETWORK_ERROR),
+			!g_error_matches (mapi_error, E_MAPI_ERROR, ecRpcFailed),
 			NULL, NULL);
 		g_object_unref (ebma->priv->conn);
 		ebma->priv->conn = NULL;
@@ -539,12 +546,14 @@ ebbm_open (EBookBackendMAPI *ebma,
 }
 
 static ESourceAuthenticationResult
-ebbm_try_password_sync (ESourceAuthenticator *authenticator,
-			const GString *password,
+ebbm_authenticate_sync (EBackend *backend,
+			const ENamedParameters *credentials,
+			gchar **out_certificate_pem,
+			GTlsCertificateFlags *out_certificate_errors,
 			GCancellable *cancellable,
 			GError **error)
 {
-	return ebbm_connect_user (E_BOOK_BACKEND_MAPI (authenticator), cancellable, password, error);
+	return ebbm_connect_user (E_BOOK_BACKEND_MAPI (backend), credentials, FALSE, cancellable, error);
 }
 
 static void
@@ -1400,6 +1409,7 @@ e_book_backend_mapi_class_init (EBookBackendMAPIClass *klass)
 	object_class->dispose			= ebbm_dispose;
 
 	backend_class->get_destination_address	= ebbm_get_destination_address;
+	backend_class->authenticate_sync	= ebbm_authenticate_sync;
 
 	book_backend_class->open		= ebbm_op_open;
 	book_backend_class->create_contacts	= ebbm_op_create_contacts;
@@ -1424,12 +1434,6 @@ e_book_backend_mapi_class_init (EBookBackendMAPIClass *klass)
 	klass->op_transfer_contacts		= NULL;
 }
 
-static void
-e_book_backend_mapi_authenticator_init (ESourceAuthenticatorInterface *iface)
-{
-	iface->try_password_sync = ebbm_try_password_sync;
-}
-
 const gchar *
 e_book_backend_mapi_get_book_uid (EBookBackendMAPI *ebma)
 {
@@ -1721,7 +1725,7 @@ mapi_error_to_edb_error (GError **perror, const GError *mapi_error, EDataBookSta
 		case MAPI_E_PASSWORD_EXPIRED:
 			code = E_DATA_BOOK_STATUS_AUTHENTICATION_REQUIRED;
 			break;
-		case MAPI_E_NETWORK_ERROR:
+		case ecRpcFailed:
 			code = E_DATA_BOOK_STATUS_REPOSITORY_OFFLINE;
 			break;
 		default:
